@@ -20,6 +20,7 @@
  *
  */
 
+#include <config.h>
 #include <gtk/gtk.h>
 #include <gdk/gdk.h>
 #include <gconf/gconf-client.h>
@@ -33,25 +34,34 @@
 
 #define NOTIFICATION_VERSION "0.1"
 
+void xchat_plugin_get_info   (char **plugin_name, char **plugin_desc, char **plugin_version, void **reserved);
+int  xchat_plugin_init       (xchat_plugin *plugin_handle, char **plugin_name, char **plugin_desc, char **plugin_version, char *arg);
+int  xchat_plugin_deinit     (void);
+int  xchat_gnome_plugin_init (xchat_gnome_plugin *xg_plugin);
+static gboolean add_channels_foreach_cb (GtkTreeModel * model, GtkTreePath * path, GtkTreeIter * iter, gpointer data);
+static gboolean tray_destroy_cb (GtkObject *object, gpointer data);
+
 /* Enumerated type of different status levels. */
 typedef enum
 {
 	NOTIF_NONE = 0,
 	NOTIF_DATA,
 	NOTIF_MSG,
-	NOTIF_NICK
+	NOTIF_NICK,
+	N_NOTIF
 } NotifStatus;
 
-static EggTrayIcon*        notification;        /* Notification area icon. */
-static gboolean            focused = TRUE;      /* GTK_WIDGET_HAS_FOCUS doesn't seem to be working... */
-static gboolean            persistant;          /* Keep the icon in the tray at all times? */
-static gboolean            hidden = FALSE;      /* True when the main window is hidden. */
-static GdkPixbuf*          pixbufs[4];          /* Pixbufs */
-static GtkWidget*          image;               /* The image displayed by the icon. */
-static GtkWidget*          main_window;         /* xchat-gnome's main window. */
-static NotifStatus         status = NOTIF_NONE; /* Current status level. */
-static xchat_gnome_plugin* xgph;                /* xchat gnome plugin handle. */
-static xchat_plugin*       ph;                  /* Plugin handle. */
+static EggTrayIcon*        notification;         /* Notification area icon. */
+static gboolean            focused = TRUE;       /* GTK_WIDGET_HAS_FOCUS doesn't seem to be working... */
+static gboolean            persistant;           /* Keep the icon in the tray at all times? */
+static gboolean            hidden = FALSE;       /* True when the main window is hidden. */
+static GdkPixbuf*          pixbufs[N_NOTIF];     /* Pixbufs */
+static GtkWidget*          image;                /* The image displayed by the icon. */
+static GtkWidget*          main_window;          /* xchat-gnome's main window. */
+static NotifStatus         status = NOTIF_NONE;  /* Current status level. */
+static xchat_gnome_plugin* xgph;                 /* xchat gnome plugin handle. */
+static xchat_plugin*       ph;                   /* Plugin handle. */
+static guint               tray_destroy_handler; /* Signal handler to recreate widget if the notification area disappears */
 
 /*** Callbacks ***/
 static gboolean
@@ -66,7 +76,7 @@ got_focus_cb (GtkWidget * widget, GdkEventFocus * event, gpointer data)
 		/* Hide the notification icon. */
 		gtk_widget_hide_all (GTK_WIDGET (notification));
 	} else {
-		gtk_image_set_from_pixbuf (GTK_IMAGE (image), pixbufs[0]);
+		gtk_image_set_from_pixbuf (GTK_IMAGE (image), pixbufs[NOTIF_NONE]);
 
 		/* Show the notification icon. */
 		gtk_widget_show_all (GTK_WIDGET (notification));
@@ -85,7 +95,7 @@ lost_focus_cb (GtkWidget * widget, GdkEventFocus * event, gpointer data)
 static int
 new_msg_cb (char **word, void *msg_lvl)
 {
-	if (status <= (NotifStatus) msg_lvl && !focused) {
+	if (status <= (NotifStatus) msg_lvl && (NotifStatus) msg_lvl < N_NOTIF && !focused) {
 		status = (NotifStatus) msg_lvl;
 		gtk_image_set_from_pixbuf (GTK_IMAGE (image), pixbufs[status]);
 		gtk_widget_show_all (GTK_WIDGET (notification));
@@ -103,8 +113,10 @@ notification_clicked_cb (GtkWidget * widget, GdkEventButton * event, gpointer da
 			if (persistant) {
 				if (hidden) {
 					xchat_command (ph, "GUI SHOW");
+					hidden = FALSE;
 				} else {
 					xchat_command (ph, "GUI HIDE");
+					hidden = TRUE;
 				}
 
 				hidden = !hidden;
@@ -123,7 +135,7 @@ notification_clicked_cb (GtkWidget * widget, GdkEventButton * event, gpointer da
 
 
 /*** Utility Functions ***/
-gboolean
+static gboolean
 add_channels_foreach_cb (GtkTreeModel * model, GtkTreePath * path, GtkTreeIter * iter, gpointer data)
 {
 	gchar* channel;
@@ -138,8 +150,8 @@ add_channels_foreach_cb (GtkTreeModel * model, GtkTreePath * path, GtkTreeIter *
 void
 xchat_plugin_get_info (char **plugin_name, char **plugin_desc, char **plugin_version, void **reserved)
 {
-	*plugin_name = "Notification";
-	*plugin_desc = "A notification area plugin.";
+	*plugin_name = _("Notification");
+	*plugin_desc = _("A notification area plugin.");
 	*plugin_version = NOTIFICATION_VERSION;
 
 	if (reserved)
@@ -166,8 +178,7 @@ xchat_gnome_plugin_init (xchat_gnome_plugin * xg_plugin)
 int
 xchat_plugin_init (xchat_plugin * plugin_handle, char **plugin_name, char **plugin_desc, char **plugin_version, char *arg)
 {
-	GtkWidget   *box;
-	GdkPixbuf   *p;
+	GdkPixbuf   *icon, *newdata, *global, *nicksaid;
 	GConfClient *client = gconf_client_get_default ();
 
 	ph = plugin_handle;
@@ -183,39 +194,25 @@ xchat_plugin_init (xchat_plugin * plugin_handle, char **plugin_name, char **plug
 	 */
 
 	/* Load the pixbufs. */
-	/* xchat-gnome logo. */
-	p = gdk_pixbuf_new_from_file (XCHATSHAREDIR "/xchat-gnome-small.png", 0);
-	pixbufs[0] = gdk_pixbuf_scale_simple (p, 16, 16, GDK_INTERP_BILINEAR);
+	if (g_file_test ("../../data/xchat-gnome-small.png", G_FILE_TEST_EXISTS)) {
+		icon     = gdk_pixbuf_new_from_file ("../../data/xchat-gnome.png", 0);
+		newdata  = gdk_pixbuf_new_from_file ("../../data/newdata.png", 0);
+		global   = gdk_pixbuf_new_from_file ("../../data/global-message.png", 0);
+		nicksaid = gdk_pixbuf_new_from_file ("../../data/nicksaid.png", 0);
+	} else {
+		icon     = gdk_pixbuf_new_from_file (XCHATSHAREDIR "/xchat-gnome.png", 0);
+		newdata  = gdk_pixbuf_new_from_file (XCHATSHAREDIR "/newdata.png", 0);
+		global   = gdk_pixbuf_new_from_file (XCHATSHAREDIR "/global-message.png", 0);
+		nicksaid = gdk_pixbuf_new_from_file (XCHATSHAREDIR "/nicksaid.png", 0);
+	}
 
-	/* New data image. */
-	p = gdk_pixbuf_new_from_file (XCHATSHAREDIR "/newdata.png", 0);
-	pixbufs[1] = gdk_pixbuf_scale_simple (p, 16, 16, GDK_INTERP_BILINEAR);
-
-	/* New message image. */
-	p = gdk_pixbuf_new_from_file (XCHATSHAREDIR "/global-message.png", 0);
-	pixbufs[2] = gdk_pixbuf_scale_simple (p, 16, 16, GDK_INTERP_BILINEAR);
-
-	/* Nick said image. */
-	p = gdk_pixbuf_new_from_file (XCHATSHAREDIR "/nicksaid.png", 0);
-	pixbufs[3] = gdk_pixbuf_scale_simple (p, 16, 16, GDK_INTERP_BILINEAR);
+	pixbufs[NOTIF_NONE] = gdk_pixbuf_scale_simple (icon,     16, 16, GDK_INTERP_BILINEAR);
+	pixbufs[NOTIF_DATA] = gdk_pixbuf_scale_simple (newdata,  16, 16, GDK_INTERP_BILINEAR);
+	pixbufs[NOTIF_MSG]  = gdk_pixbuf_scale_simple (global,   16, 16, GDK_INTERP_BILINEAR);
+	pixbufs[NOTIF_NICK] = gdk_pixbuf_scale_simple (nicksaid, 16, 16, GDK_INTERP_BILINEAR);
 
 	/* Create the notification icon. */
-	notification = egg_tray_icon_new ("xchat-gnome");
-	box = gtk_event_box_new ();
-	image = gtk_image_new_from_pixbuf (pixbufs[0]);
-
-	g_signal_connect (G_OBJECT (box), "button-press-event", G_CALLBACK (notification_clicked_cb), NULL);
-
-	gtk_container_add (GTK_CONTAINER (box), image);
-	gtk_container_add (GTK_CONTAINER (notification), box);
-
-	gtk_widget_show_all (GTK_WIDGET (notification));
-
-	/* FIXME: Saw this in Gaim's notification plugin. Not sure it's necessary,
-	 *        will require investigation at a later date.
-	 */
-	g_object_ref (G_OBJECT (notification));
-
+	tray_destroy_cb (NULL, NULL);
 
 	/* Hook up our callbacks. */
 	xchat_hook_print (ph, "Channel Notice",			XCHAT_PRI_NORM, new_msg_cb, (gpointer) NOTIF_DATA);
@@ -223,24 +220,54 @@ xchat_plugin_init (xchat_plugin * plugin_handle, char **plugin_name, char **plug
 	xchat_hook_print (ph, "Channel Action",			XCHAT_PRI_NORM, new_msg_cb, (gpointer) NOTIF_MSG);
 	xchat_hook_print (ph, "Channel Msg Hilight",		XCHAT_PRI_NORM, new_msg_cb, (gpointer) NOTIF_NICK);
 	xchat_hook_print (ph, "Channel Action Hilight",		XCHAT_PRI_NORM, new_msg_cb, (gpointer) NOTIF_NICK);
-	xchat_hook_print (ph, "Private Message to Dialog",	XCHAT_PRI_NORM, new_msg_cb, (gpointer) NOTIF_MSG);
+	xchat_hook_print (ph, "Private Message to Dialog",	XCHAT_PRI_NORM, new_msg_cb, (gpointer) NOTIF_NICK);
 
-	xchat_print (ph, "Notification plugin loaded.\n");
+	xchat_print (ph, _("Notification plugin loaded.\n"));
 
 	return TRUE;
 }
 
+static gboolean
+tray_destroy_cb (GtkObject *object,
+		 gpointer data)
+{
+	GtkWidget   *box;
+
+	if (notification) {
+		gtk_object_sink (GTK_OBJECT (notification));
+		notification = NULL;
+	}
+
+	notification = egg_tray_icon_new ("xchat-gnome");
+	box = gtk_event_box_new ();
+	image = gtk_image_new_from_pixbuf (pixbufs[NOTIF_NONE]);
+
+	g_signal_connect (G_OBJECT (box), "button-press-event", G_CALLBACK (notification_clicked_cb), NULL);
+
+	gtk_container_add (GTK_CONTAINER (box), image);
+	gtk_container_add (GTK_CONTAINER (notification), box);
+
+	tray_destroy_handler = g_signal_connect_object (G_OBJECT (notification), "destroy", G_CALLBACK (tray_destroy_cb), data, 0);
+
+ 	gtk_widget_show_all (GTK_WIDGET (notification));
+ 	return TRUE;
+}
+
+
 int
-xchat_plugin_deinit ()
+xchat_plugin_deinit (void)
 {
 	/* Disconnect the signal handlers. */
 	g_signal_handlers_disconnect_by_func (main_window, G_CALLBACK (got_focus_cb), NULL);
 	g_signal_handlers_disconnect_by_func (main_window, G_CALLBACK (lost_focus_cb), NULL);
 
-	g_object_unref (G_OBJECT (notification));
-	gtk_widget_destroy (GTK_WIDGET (notification));
+	if (notification) {
+		g_signal_handler_disconnect (G_OBJECT(notification), tray_destroy_handler);
+		gtk_widget_destroy (GTK_WIDGET (notification));
+		notification = NULL;
+	}
 
-	xchat_print (ph, "Notification plugin unloaded.\n");
+	xchat_print (ph, _("Notification plugin unloaded.\n"));
 
 	return 1;
 }
