@@ -1,7 +1,7 @@
 /*
  * main-window.c - main GUI window functions
  *
- * Copyright (C) 2004-2005 xchat-gnome team
+ * Copyright (C) 2004-2006 xchat-gnome team
  *
  * This program is free software; you can redistribute it and/or
  * modify it under the terms of the GNU General Public License
@@ -92,6 +92,8 @@ static void on_add_widget (GtkUIManager *manager, GtkWidget *menu, GtkWidget *me
 static gboolean on_resize (GtkWidget *widget, GdkEventConfigure *event, gpointer data);
 static gboolean on_hpane_move (GtkPaned *widget, GParamSpec *param_spec, gpointer data);
 
+static gboolean on_main_window_focus_in (GtkWidget *widget, GdkEventFocus *event, gpointer data);
+
 static GtkActionEntry action_entries [] = {
 
 	/* Toplevel */
@@ -153,6 +155,8 @@ initialize_main_window (void)
 	gui.main_window = glade_xml_get_widget (gui.xml, "xchat-gnome");
 	g_signal_connect (G_OBJECT (gui.main_window), "delete-event",
 	                  G_CALLBACK (on_main_window_close), NULL);
+	g_signal_connect (G_OBJECT (gui.main_window), "focus-in-event",
+	                  G_CALLBACK (on_main_window_focus_in), NULL);
 
 	/* hook up the menus */
 	gui.action_group = gtk_action_group_new ("MenuAction");
@@ -329,7 +333,10 @@ rename_main_window (gchar *server, gchar *channel)
 	gchar *new_title;
 
 	if (server == NULL) {
-		gtk_window_set_title (GTK_WINDOW (gui.main_window), channel);
+		if (channel && strlen (channel) != 0)
+			gtk_window_set_title (GTK_WINDOW (gui.main_window), channel);
+		else
+			gtk_window_set_title (GTK_WINDOW (gui.main_window), "XChat-GNOME");
 		return;
 	}
 	new_title = g_strconcat (server, ": ", channel, NULL);
@@ -576,7 +583,7 @@ on_help_contents_activate (GtkAction *action, gpointer data)
 {
 	GError *error = NULL;
 
-	gnome_help_display_with_doc_id (NULL, NULL, "xchat-gnome.xml", NULL, &error);
+	gnome_help_display_with_doc_id (NULL, "xchat-gnome", "xchat-gnome.xml", NULL, &error);
 	if (error) {
 		error_dialog (_("Error showing help"), error->message);
 		g_error_free (error);
@@ -593,11 +600,15 @@ static void
 nickname_get_str_response (GtkDialog *dialog, gint arg1, gpointer entry)
 {
 	gchar *text, *buf;
-
-	text = (gchar *) gtk_entry_get_text (GTK_ENTRY (entry));
+	GtkWidget *check;
 
 	if (arg1 == GTK_RESPONSE_ACCEPT || arg1 == GTK_RESPONSE_OK) {
-		buf = g_strdup_printf ("nick %s", text);
+		text = (gchar *) gtk_entry_get_text (GTK_ENTRY (entry));
+		check = glade_xml_get_widget (gui.xml, "nickname dialog check");
+		if (gtk_toggle_button_get_active (GTK_TOGGLE_BUTTON (check)))
+			buf = g_strdup_printf ("allserv nick %s", text);
+		else
+			buf = g_strdup_printf ("nick %s", text);
 		handle_command (current_sess, buf, FALSE);
 		g_free (buf);
 	}
@@ -616,15 +627,18 @@ on_nickname_clicked (GtkButton *widget, gpointer user_data)
 	GtkWidget *dialog;
 	GtkWidget *entry;
 
-	if (!current_sess)
+	if (gui.current_session == NULL)
 		return;
+	current_sess = gui.current_session;
 
 	dialog = glade_xml_get_widget (gui.xml, "nickname dialog");
 	entry = glade_xml_get_widget (gui.xml, "nickname dialog entry");
 
-	gtk_entry_set_text (GTK_ENTRY (entry), current_sess->server->nick); g_signal_connect (G_OBJECT (entry), "activate",
+	gtk_entry_set_text (GTK_ENTRY (entry), current_sess->server->nick);
+	g_signal_connect (G_OBJECT (entry), "activate",
 	                  G_CALLBACK (nickname_str_enter), dialog);
 	g_signal_connect (G_OBJECT (dialog), "response", G_CALLBACK (nickname_get_str_response), entry);
+	g_signal_connect (G_OBJECT (dialog), "delete-event", G_CALLBACK (gtk_widget_hide_on_delete), NULL);
 
 	gtk_widget_show_all (dialog);
 }
@@ -682,5 +696,41 @@ set_nickname (struct server *serv, char *newnick)
 			gtk_button_set_label (GTK_BUTTON (nick), serv->nick);
 		else
 			gtk_button_set_label (GTK_BUTTON (nick), newnick);
+		set_nickname_color (serv);
 	}
 }
+
+void
+set_nickname_color (struct server *serv)
+{
+	if (serv == gui.current_session->server) {
+		GtkWidget *nick_button;
+		GtkLabel *label;
+		PangoAttribute *attr;
+		PangoAttrList *l;
+
+		l = pango_attr_list_new ();
+		nick_button = glade_xml_get_widget (gui.xml, "nickname");
+		label = GTK_LABEL (GTK_BIN (nick_button)->child);
+
+		if (serv->is_away)
+			attr = pango_attr_foreground_new (0x9999, 0x9999, 0x9999);
+		else
+			/* FIXME: maybe we should get the color from GTK theme */
+			attr = pango_attr_foreground_new (0x0000, 0x0000, 0x0000);
+
+		attr->start_index = 0;
+		attr->end_index = G_MAXUINT;
+		pango_attr_list_insert (l, attr);
+		gtk_label_set_attributes (label, l);
+
+		pango_attr_list_unref (l);
+	}
+}
+
+static gboolean
+on_main_window_focus_in (GtkWidget * widget, GdkEventFocus * event, gpointer data)
+{
+	gtk_window_set_urgency_hint (GTK_WINDOW (widget), FALSE);
+	return FALSE;
+}
