@@ -16,6 +16,9 @@
  * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA
  */
 
+#include <config.h>
+#include <glib/gi18n.h>
+#include <string.h>
 #include <stdio.h>
 #include <stdlib.h>
 #include <unistd.h>
@@ -23,6 +26,7 @@
 #include <sys/stat.h>
 #include <fcntl.h>
 #include <gnome.h>
+#include "palette.h"
 #include "preferences.h"
 #include "../common/xchat.h"
 #include "../common/util.h"
@@ -65,45 +69,43 @@ GdkColor colors[] =
 	{0, 0x7f7f, 0x7f7f, 0x7f7f}, /* 30 grey */
 	{0, 0x9595, 0x9595, 0x9595}, /* 31 light grey */
 
-	{0, 0x0000, 0x0000, 0x0000}, /* 32 marktext Fore (black) */
-	{0, 0xa4a4, 0xdfdf, 0xffff}, /* 33 marktext Back (blue) */
-	{0, 0xdf3c, 0xdf3c, 0xdf3c}, /* 34 foreground (white) */
-	{0, 0x0000, 0x0000, 0x0000}, /* 35 background (black) */
-	{0, 0xcccc, 0x0000, 0x0000}, /* 36 marker line (red) */
-
-	/* colors for GUI */
-	{0, 0x8c8c, 0x1010, 0x1010}, /* 37 tab New Data (dark red) */
-	{0, 0x0000, 0x0000, 0xffff}, /* 38 tab Nick Mentioned (blue) */
-	{0, 0xf5f5, 0x0000, 0x0000}, /* 39 tab New Message (red) */
-	{0, 0x9999, 0x9999, 0x9999}, /* 40 away user (grey) */
+	{0, 0x0000, 0x0000, 0x0000}, /* 32 marktext Fore (black)     - 256 */
+	{0, 0xa4a4, 0xdfdf, 0xffff}, /* 33 marktext Back (blue)      - 257 */
+	{0, 0xdf3c, 0xdf3c, 0xdf3c}, /* 34 foreground (white)        - 258 */
+	{0, 0x0000, 0x0000, 0x0000}, /* 35 background (black)        - 259 */
+	{0, 0xcccc, 0x0000, 0x0000}, /* 36 marker line (red)         - 260 */
+	{0, 0x8c8c, 0x1010, 0x1010}, /* 37 tab New Data (dark red)   - 261 */
+	{0, 0x0000, 0x0000, 0xffff}, /* 38 tab Nick Mentioned (blue) - 262 */
+	{0, 0xf5f5, 0x0000, 0x0000}, /* 39 tab New Message (red)     - 263 */
+	{0, 0x9999, 0x9999, 0x9999}, /* 40 away user (grey)          - 264 */
 };
 
 #define MAX_COL 40
 
 const GdkColor colors_white_on_black[] =
 {
-	{0, 0xffff, 0xffff, 0xffff}, /* background (white) */
-	{0, 0x0000, 0x0000, 0x0000}, /* foreground (black) */
-	{0, 0x0000, 0x0000, 0x0000}, /* marktext fore (black) */
+	{0, 0xdf3c, 0xdf3c, 0xdf3c}, /* foreground (white) */
+	{0, 0x0000, 0x0000, 0x0000}, /* background (black) */
+	{0, 0xdf3c, 0xdf3c, 0xdf3c}, /* marktext fore (white) */
 	{0, 0xa4a4, 0xdfdf, 0xffff}, /* marktext back (blue) */
 	{0, 0x9999, 0x9999, 0x9999}, /* away user (grey) */
 };
 
 const GdkColor colors_black_on_white[] =
 {
-	{0, 0x0000, 0x0000, 0x0000}, /* background (black) */
-	{0, 0xdf3c, 0xdf3c, 0xdf3c}, /* foreground (white) */
-	{0, 0xdf3c, 0xdf3c, 0xdf3c}, /* marktext fore (white) */
+	{0, 0x0000, 0x0000, 0x0000}, /* foreground (black) */
+	{0, 0xffff, 0xffff, 0xffff}, /* background (white) */
+	{0, 0x0000, 0x0000, 0x0000}, /* marktext fore (black) */
 	{0, 0xa4a4, 0xdfdf, 0xffff}, /* marktext back (blue) */
 	{0, 0x9999, 0x9999, 0x9999}, /* away user (grey) */
 };
 
-GdkColor custom_colors[5];
+GdkColor custom_colors[9];
 
 const GdkColor *color_schemes[] =
 {
-	colors_white_on_black,
 	colors_black_on_white,
+	colors_white_on_black,
 	custom_colors,
 };
 
@@ -193,8 +195,8 @@ const GdkColor *palette_schemes[] =
 void
 load_colors (int selection)
 {
-	colors[35] = color_schemes[selection][0];
-	colors[34] = color_schemes[selection][1];
+	colors[34] = color_schemes[selection][0];
+	colors[35] = color_schemes[selection][1];
 	colors[32] = color_schemes[selection][2];
 	colors[33] = color_schemes[selection][3];
 	colors[40] = color_schemes[selection][4];
@@ -210,7 +212,7 @@ load_palette (int selection)
 }
 
 void
-palette_init ()
+palette_init (void)
 {
 	int i, f, l;
 	int red, green, blue;
@@ -244,13 +246,13 @@ palette_init ()
 			custom_palette[i].blue = blue;
 		}
 
-		strcpy (prefname, "color_259");
+		strcpy (prefname, "color_258");
 		cfg_get_color (cfg, prefname, &red, &green, &blue);
 		custom_colors[0].red = red;
 		custom_colors[0].green = green;
 		custom_colors[0].blue = blue;
 
-		strcpy (prefname, "color_258");
+		strcpy (prefname, "color_259");
 		cfg_get_color (cfg, prefname, &red, &green, &blue);
 		custom_colors[1].red = red;
 		custom_colors[1].green = green;
@@ -274,13 +276,37 @@ palette_init ()
 		custom_colors[4].green = green;
 		custom_colors[4].blue = blue;
 
+		strcpy (prefname, "color_261");
+		cfg_get_color (cfg, prefname, &red, &green, &blue);
+		custom_colors[5].red = red;
+		custom_colors[5].green = green;
+		custom_colors[5].blue = blue;
+
+		strcpy (prefname, "color_262");
+		cfg_get_color (cfg, prefname, &red, &green, &blue);
+		custom_colors[6].red = red;
+		custom_colors[6].green = green;
+		custom_colors[6].blue = blue;
+
+		strcpy (prefname, "color_263");
+		cfg_get_color (cfg, prefname, &red, &green, &blue);
+		custom_colors[7].red = red;
+		custom_colors[7].green = green;
+		custom_colors[7].blue = blue;
+
+		strcpy (prefname, "color_260");
+		cfg_get_color (cfg, prefname, &red, &green, &blue);
+		custom_colors[8].red = red;
+		custom_colors[8].green = green;
+		custom_colors[8].blue = blue;
+
 		free (cfg);
 	}
 	close (f);
 }
 
 void
-palette_save ()
+palette_save (void)
 {
 	int i, f;
 	char prefname[256];
@@ -293,20 +319,15 @@ palette_save ()
 			cfg_put_color (f, custom_palette[i].red, custom_palette[i].green, custom_palette[i].blue, prefname);
 		}
 
-		strcpy (prefname, "color_259");
-		cfg_put_color (f, custom_colors[0].red, custom_colors[0].green, custom_colors[0].blue, prefname);
-
-		strcpy (prefname, "color_258");
-		cfg_put_color (f, custom_colors[1].red, custom_colors[1].green, custom_colors[1].blue, prefname);
-
-		strcpy (prefname, "color_256");
-		cfg_put_color (f, custom_colors[2].red, custom_colors[2].green, custom_colors[2].blue, prefname);
-
-		strcpy (prefname, "color_257");
-		cfg_put_color (f, custom_colors[3].red, custom_colors[3].green, custom_colors[3].blue, prefname);
-
-		strcpy (prefname, "color_264");
-		cfg_put_color (f, custom_colors[4].red, custom_colors[4].green, custom_colors[4].blue, prefname);
+		cfg_put_color (f, custom_colors[0].red, custom_colors[0].green, custom_colors[0].blue, "color_258");
+		cfg_put_color (f, custom_colors[1].red, custom_colors[1].green, custom_colors[1].blue, "color_259");
+		cfg_put_color (f, custom_colors[2].red, custom_colors[2].green, custom_colors[2].blue, "color_256");
+		cfg_put_color (f, custom_colors[3].red, custom_colors[3].green, custom_colors[3].blue, "color_257");
+		cfg_put_color (f, custom_colors[4].red, custom_colors[4].green, custom_colors[4].blue, "color_264");
+		cfg_put_color (f, custom_colors[5].red, custom_colors[5].green, custom_colors[5].blue, "color_261");
+		cfg_put_color (f, custom_colors[6].red, custom_colors[6].green, custom_colors[6].blue, "color_262");
+		cfg_put_color (f, custom_colors[7].red, custom_colors[7].green, custom_colors[7].blue, "color_263");
+		cfg_put_color (f, custom_colors[8].red, custom_colors[8].green, custom_colors[8].blue, "color_260");
 
 		close (f);
 	}
