@@ -1,7 +1,7 @@
 /*
  * preferences-page-irc.c - helpers for the irc preferences page
  *
- * Copyright (C) 2004-2005 xchat-gnome team
+ * Copyright (C) 2004-2006 xchat-gnome team
  *
  * This program is free software; you can redistribute it and/or
  * modify it under the terms of the GNU General Public License
@@ -127,7 +127,7 @@ save_highlight (PreferencesIrcPage *page)
 		gtk_tree_model_get (GTK_TREE_MODEL (page->highlight_store), &iter, 0, &tmp1, -1);
 		highlight = g_strdup (tmp1);
 	} else {
-		prefs.irc_nick_hilight[0] = '\0';
+		prefs.irc_extra_hilight[0] = '\0';
 		return;
 	}
 	while (gtk_tree_model_iter_next (GTK_TREE_MODEL (page->highlight_store), &iter)) {
@@ -136,7 +136,7 @@ save_highlight (PreferencesIrcPage *page)
 		highlight = g_strdup_printf ("%s,%s", tmp2, tmp1);
 		g_free (tmp2);
 	}
-	strncpy (prefs.irc_nick_hilight, highlight, 300);
+	strncpy (prefs.irc_extra_hilight, highlight, 300);
 	g_free (highlight);
 }
 
@@ -199,6 +199,22 @@ highlight_edited (GtkCellRendererText *renderer, gchar *arg1, gchar *newtext, Pr
 	}
 }
 
+static void
+highlight_canceled (GtkCellRendererText *renderer, PreferencesIrcPage *page)
+{
+	GtkTreeSelection *selection;
+	GtkTreeModel *model;
+	GtkTreeIter iter;
+	gchar *text;
+
+	selection = gtk_tree_view_get_selection (GTK_TREE_VIEW (page->highlight_list));
+	if (gtk_tree_selection_get_selected (selection, &model, &iter)) {
+		gtk_tree_model_get (GTK_TREE_MODEL (page->highlight_store), &iter, 0, &text, -1);
+		if (text == NULL)
+			gtk_list_store_remove (GTK_LIST_STORE (model), &iter);
+	}
+}
+
 PreferencesIrcPage *
 preferences_page_irc_new (gpointer prefs_dialog, GladeXML *xml)
 {
@@ -274,24 +290,44 @@ preferences_page_irc_new (gpointer prefs_dialog, GladeXML *xml)
 	page->nh[8] = gconf_client_notify_add (p->gconf, "/apps/xchat/irc/showtimestamps",        (GConfClientNotifyFunc) gconf_bool_changed,  page->show_timestamps, NULL, NULL);
 
 	text = gconf_client_get_string (p->gconf, "/apps/xchat/irc/nickname", NULL);
-	gtk_entry_set_text (GTK_ENTRY (page->nick_name), text);
-	g_free (text);
+	if (text) {
+		gtk_entry_set_text (GTK_ENTRY (page->nick_name), text);
+		g_free (text);
+	} else {
+		gtk_entry_set_text (GTK_ENTRY (page->nick_name), "");
+	}
 
 	text = gconf_client_get_string (p->gconf, "/apps/xchat/irc/realname", NULL);
-	gtk_entry_set_text (GTK_ENTRY (page->real_name), text);
-	g_free (text);
+	if (text) {
+		gtk_entry_set_text (GTK_ENTRY (page->real_name), text);
+		g_free (text);
+	} else {
+		gtk_entry_set_text (GTK_ENTRY (page->real_name), "");
+	}
 
 	text = gconf_client_get_string (p->gconf, "/apps/xchat/irc/quitmsg", NULL);
-	gtk_entry_set_text (GTK_ENTRY (page->quit_message), text);
-	g_free (text);
+	if (text) {
+		gtk_entry_set_text (GTK_ENTRY (page->quit_message), text);
+		g_free (text);
+	} else {
+		gtk_entry_set_text (GTK_ENTRY (page->quit_message), "");
+	}
 
 	text = gconf_client_get_string (p->gconf, "/apps/xchat/irc/partmsg", NULL);
-	gtk_entry_set_text (GTK_ENTRY (page->part_message), text);
-	g_free (text);
+	if (text) {
+		gtk_entry_set_text (GTK_ENTRY (page->part_message), text);
+		g_free (text);
+	} else {
+		gtk_entry_set_text (GTK_ENTRY (page->part_message), "");
+	}
 
 	text = gconf_client_get_string (p->gconf, "/apps/xchat/irc/awaymsg", NULL);
-	gtk_entry_set_text (GTK_ENTRY (page->away_message), text);
-	g_free (text);
+	if (text) {
+		gtk_entry_set_text (GTK_ENTRY (page->away_message), text);
+		g_free (text);
+	} else {
+		gtk_entry_set_text (GTK_ENTRY (page->away_message), "");
+	}
 
 	toggle = gconf_client_get_bool (p->gconf, "/apps/xchat/main_window/use_sys_fonts", NULL);
 	if (toggle)
@@ -301,8 +337,10 @@ preferences_page_irc_new (gpointer prefs_dialog, GladeXML *xml)
 	gtk_widget_set_sensitive (page->font_selection, !toggle);
 
 	text = gconf_client_get_string (p->gconf, "/apps/xchat/main_window/font", NULL);
-	gtk_font_button_set_font_name (GTK_FONT_BUTTON (page->font_selection), text);
-	g_free (text);
+	if (text) {
+		gtk_font_button_set_font_name (GTK_FONT_BUTTON (page->font_selection), text);
+		g_free (text);
+	}
 
 	toggle = gconf_client_get_bool (p->gconf, "/apps/xchat/irc/showcolors", NULL);
 	gtk_toggle_button_set_active (GTK_TOGGLE_BUTTON (page->show_colors), toggle);
@@ -322,8 +360,9 @@ preferences_page_irc_new (gpointer prefs_dialog, GladeXML *xml)
 	select = gtk_tree_view_get_selection (GTK_TREE_VIEW (page->highlight_list));
 	g_signal_connect (G_OBJECT (select), "changed", G_CALLBACK (highlight_selection_changed), page);
 	g_signal_connect (G_OBJECT (renderer), "edited", G_CALLBACK (highlight_edited), page);
+	g_signal_connect (G_OBJECT (renderer), "editing-canceled", G_CALLBACK (highlight_canceled), page);
 
-	highlight_entries = g_strsplit (prefs.irc_nick_hilight, ",", 0);
+	highlight_entries = g_strsplit (prefs.irc_extra_hilight, ",", 0);
 	for (i = 0; highlight_entries[i]; i++) {
 		gtk_list_store_append (page->highlight_store, &iter);
 		gtk_list_store_set (page->highlight_store, &iter, 0, highlight_entries[i], -1);
